from kfsd.apps.core.common.configuration import Configuration


class Config:
    def __init__(self, request):
        self.__request = request

    def getConfigData(self):
        return self.__request.config.getFinalConfig()

    def findConfigs(self, paths):
        return Configuration.findConfigValues(
            self.getConfigData(),
            paths
        )



from kfsd.apps.core.utils.http.cookie import Cookie
from kfsd.apps.core.utils.http.config import Config


class Request(Cookie, Config):
    def __init__(self, request=None):
        self.__request = request
        Cookie.__init__(self, request)
        Config.__init__(self, request)

    def getRequest(self):
        return self.__request

    def setRequest(self, request):
        self.__request = request



from django.test import TestCase
from django.urls import reverse


class APIDocsViewTests(TestCase):

    def test_get(self):
        url = reverse('api_doc')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from kfsd.apps.core.auth.api.gateway import APIGateway


class TokenAuth(APIGateway):
    def __init__(self, request=None, response=None):
        APIGateway.__init__(self, request, response)

    def getTokenUserInfo(self):
        pass
        # uri = self.findConfigs(["gateway.auth.token.verify_token"])[0]
        # headers = {
        #     'Content-Type': 'application/json',
        #     'X-CSRFToken': self.getCookie('csrftoken'),
        #     'Cookie': self.cookieToHeaderStr(),
        #     'X-APIKey': self.getApplicationAPIKey()
        # }
        # self.post(uri, HTTPStatus.OK, headers=headers)



import requests
from rest_framework import status

from kfsd.apps.core.exceptions.api import KubefacetsAPIException


class Response():
    def __init__(self, response):
        self.__response = response

    def getResponse(self):
        return self.__response

    def setResponse(self, response):
        self.__response = response

    def getStatusCode(self):
        return self.getResponse().status_code

    def isRespValid(self, expStatusCode):
        if isinstance(expStatusCode, int) and not expStatusCode == self.getStatusCode():
            raise KubefacetsAPIException(
                self.getResponse().json()["detail"], self.getResponse().json()["code"], self.getStatusCode()
            )

        if isinstance(expStatusCode, list) and self.getStatusCode() not in expStatusCode:
            raise KubefacetsAPIException(
                self.getResponse().json()["detail"], self.getResponse().json()["code"], self.getStatusCode()
            )

        return True

    def request(self, method, url, expStatus, **kwargs):
        try:
            resp = method(url, **kwargs)
            if self.isRespValid(expStatus, resp.status_code, resp):
                return resp
        except requests.exceptions.Timeout:
            raise KubefacetsAPIException(
                "The server took too long to respond to your request. Please try again later.",
                "server_timed_out",
                status.HTTP_408_REQUEST_TIMEOUT
            )
        except requests.exceptions.ConnectionError:
            raise KubefacetsAPIException(
                "Service temporarily unavailable. Please try again later.",
                "service_unavailable",
                status.HTTP_503_SERVICE_UNAVAILABLE
            )

    def post(self, url, expStatus, **kwargs):
        return self.request(requests.post, url, expStatus, **kwargs)

    def get(self, url, expStatus, **kwargs):
        return self.request(requests.get, url, expStatus, **kwargs)

    def delete(self, url, expStatus, **kwargs):
        return self.request(requests.delete, url, expStatus, **kwargs)



from rest_framework import serializers


class ErrorSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()


class NotFoundSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()


class SuccessSerializer(serializers.Serializer):
    detail = serializers.CharField()
    code = serializers.CharField()



from kfsd.apps.core.auth.base import BaseUser
from kfsd.apps.core.auth.api.token import TokenAuth


class TokenUser(BaseUser, TokenAuth):
    def __init__(self, request):
        TokenAuth.__init__(self, request)
        self.setUserInfo(self.getTokenUserInfo())




class BaseUser:
    def __init__(self):
        self.__userInfo = {}

    def setUserInfo(self, userInfo):
        self.__userInfo = userInfo

    def getUserInfo(self):
        return self.__userInfo

    def is_authenticated(self):
        if not self.__userInfo:
            return False
        return True



from kfsd.apps.core.utils.http.request import Request
from kfsd.apps.core.utils.http.response import Response


class HTTP(Request, Response):
    def __init__(self, request=None, response=None):
        Request.__init__(self, request)
        Response.__init__(self, response)

    def formatUrl(self, args):
        return "/".join(args)



from kfsd.apps.core.utils.http.base import HTTP


class APIGateway(HTTP):
    def __init__(self, request=None, response=None):
        HTTP.__init__(self, request, response)

    def getApplicationAPIKey(self):
        return self.findConfigs(["gateway.api_key"])[0]

    def post(self, uri, expStatus, **kwargs):
        gatewayHost = self.findConfigs(["gateway.host"])[0]
        url = gatewayHost + uri
        super().post(url, expStatus, **kwargs)
